from django.apps import AppConfig


class PuputAppConfig(AppConfig):
    name = "puput"
    verbose_name = "Puput"
    default_auto_field = "django.db.models.AutoField"

    def ready(self):
        import puput.signals  # noqa: F401



# Generated by Django 4.0 on 2023-05-22 15:01

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('taggit', '0004_alter_taggeditem_content_type_alter_taggeditem_tag'),
        ('puput', '0007_alter_entrypage_body'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tagentrypage',
            name='tag',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='%(app_label)s_%(class)s_items', to='taggit.tag'),
        ),
    ]



import wagtail.admin.rich_text.editors.draftail.features as draftail_features
from wagtail.admin.rich_text.converters.html_to_contentstate import InlineStyleElementHandler, BlockElementHandler
from wagtail import hooks


@hooks.register("register_rich_text_features")
def register_blockquote_feature(features):
    """
    Registering the `blockquote` feature, which uses the `blockquote` Draft.js block type,
    and is stored as HTML with a `<blockquote>` tag.
    """
    feature_name = "blockquote"
    type_ = "blockquote"
    tag = "blockquote"

    control = {
        "type": type_,
        "label": "❝",
        "description": "Quote",
        "element": "blockquote",
    }

    features.register_editor_plugin("draftail", feature_name, draftail_features.BlockFeature(control))

    features.register_converter_rule(
        "contentstate",
        feature_name,
        {
            "from_database_format": {tag: BlockElementHandler(type_)},
            "to_database_format": {"block_map": {type_: tag}},
        },
    )
    features.default_features.append(feature_name)


@hooks.register("register_rich_text_features")
def register_codeline_feature(features):
    feature_name = "Code Line"
    type_ = "CODE"
    tag = "code"

    control = {
        "type": type_,
        "label": ">_",
        "description": "Code Line",
    }

    features.register_editor_plugin("draftail", feature_name, draftail_features.InlineStyleFeature(control))

    db_conversion = {
        "from_database_format": {tag: InlineStyleElementHandler(type_)},
        "to_database_format": {"style_map": {type_: tag}},
    }

    features.register_converter_rule("contentstate", feature_name, db_conversion)
    features.default_features.append(feature_name)



import datetime

from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext_lazy as _

from wagtail.admin.panels import (
    FieldPanel,
    MultiFieldPanel,
    InlinePanel,
)
from wagtail.fields import RichTextField
from modelcluster.contrib.taggit import ClusterTaggableManager
from wagtailmarkdown.fields import MarkdownField
from colorful.fields import RGBColorField

from .utils import get_image_model_path
import markdown


class BlogAbstract(models.Model):
    description = models.CharField(
        verbose_name=_("Description"),
        max_length=255,
        blank=True,
        help_text=_("The blog description that will appear under the title."),
    )
    header_image = models.ForeignKey(
        get_image_model_path(),
        verbose_name=_("Header image"),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
    )

    main_color = RGBColorField(_("Blog Main Color"), default="#4D6AE0")

    display_comments = models.BooleanField(default=False, verbose_name=_("Display comments"))
    display_categories = models.BooleanField(default=True, verbose_name=_("Display categories"))
    display_tags = models.BooleanField(default=True, verbose_name=_("Display tags"))
    display_popular_entries = models.BooleanField(default=True, verbose_name=_("Display popular entries"))
    display_last_entries = models.BooleanField(default=True, verbose_name=_("Display last entries"))
    display_archive = models.BooleanField(default=True, verbose_name=_("Display archive"))

    disqus_api_secret = models.TextField(blank=True)
    disqus_shortname = models.CharField(max_length=128, blank=True)

    num_entries_page = models.IntegerField(default=5, verbose_name=_("Entries per page"))
    num_last_entries = models.IntegerField(default=3, verbose_name=_("Last entries limit"))
    num_popular_entries = models.IntegerField(default=3, verbose_name=_("Popular entries limit"))
    num_tags_entry_header = models.IntegerField(default=5, verbose_name=_("Tags limit entry header"))

    short_feed_description = models.BooleanField(default=True, verbose_name=_("Use short description in feeds"))

    content_panels = [
        FieldPanel("description", classname="full"),
        FieldPanel("header_image"),
        FieldPanel("main_color"),
    ]
    settings_panels = [
        MultiFieldPanel(
            [
                FieldPanel("display_categories"),
                FieldPanel("display_tags"),
                FieldPanel("display_popular_entries"),
                FieldPanel("display_last_entries"),
                FieldPanel("display_archive"),
            ],
            heading=_("Widgets"),
        ),
        MultiFieldPanel(
            [
                FieldPanel("num_entries_page"),
                FieldPanel("num_last_entries"),
                FieldPanel("num_popular_entries"),
                FieldPanel("num_tags_entry_header"),
            ],
            heading=_("Parameters"),
        ),
        MultiFieldPanel(
            [
                FieldPanel("display_comments"),
                FieldPanel("disqus_api_secret"),
                FieldPanel("disqus_shortname"),
            ],
            heading=_("Comments"),
        ),
        MultiFieldPanel(
            [
                FieldPanel("short_feed_description"),
            ],
            heading=_("Feeds"),
        ),
    ]

    class Meta:
        abstract = True


class EntryAbstract(models.Model):
    body = RichTextField(verbose_name=_("body"), blank=True, null=True)
    markdown_body = MarkdownField(blank=True, null=True, verbose_name="body (Markdown)")
    tags = ClusterTaggableManager(through="puput.TagEntryPage", blank=True)
    date = models.DateTimeField(verbose_name=_("Post date"), default=datetime.datetime.today)
    header_image = models.ForeignKey(
        get_image_model_path(),
        verbose_name=_("Header image"),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
    )
    categories = models.ManyToManyField("puput.Category", through="puput.CategoryEntryPage", blank=True)
    excerpt = RichTextField(
        verbose_name=_("excerpt"),
        blank=True,
        help_text=_(
            "Entry excerpt to be displayed on entries list. "
            "If this field is not filled, a truncate version of body text will be used."
        ),
    )
    num_comments = models.IntegerField(default=0, editable=False)

    content_panels = [
        MultiFieldPanel(
            [
                FieldPanel("title", classname="title"),
                FieldPanel("header_image"),
                FieldPanel("body", classname="full"),
                FieldPanel("markdown_body"),
                FieldPanel("excerpt", classname="full"),
            ],
            heading=_("Content"),
        ),
        MultiFieldPanel(
            [
                FieldPanel("tags"),
                InlinePanel("entry_categories", label=_("Categories")),
                InlinePanel(
                    "related_entrypage_from",
                    label=_("Related Entries"),
                    panels=[FieldPanel("entrypage_to")],
                ),
            ],
            heading=_("Metadata"),
        ),
    ]

    class Meta:
        abstract = True

    def clean(self):
        if not (self.markdown_body or self.body):
            raise ValidationError("You must specify either body or markdown body")

    def save(self, *args, **kwargs):
        if self.markdown_body:
            self.body = markdown.markdown(self.markdown_body, extensions=["extra", "codehilite"])
        super().save(*args, **kwargs)



from datetime import date

from django.contrib.auth import get_user_model
from django.utils.dateformat import DateFormat
from django.utils.formats import date_format
from django.utils.translation import gettext_lazy as _
from django.conf import settings

from wagtail.contrib.routable_page.models import RoutablePageMixin, route
from wagtail.models import Page
from wagtail.search.models import Query

from .utils import get_object_or_None

USERNAME_REGEX = getattr(settings, "PUPUT_USERNAME_REGEX", "\\w+")
USERNAME_FIELD = getattr(settings, "PUPUT_USERNAME_FIELD", "username")


class BlogRoutes(RoutablePageMixin):
    @route(r"^(\d{4})/$")
    @route(r"^(\d{4})/(\d{2})/$")
    @route(r"^(\d{4})/(\d{2})/(\d{2})/$")
    def entries_by_date(self, request, year, month=None, day=None, *args, **kwargs):
        self.entries = self.get_entries().filter(date__year=year)
        self.search_type = _("date")
        self.search_term = year
        if month:
            self.entries = self.entries.filter(date__month=month)
            df = DateFormat(date(int(year), int(month), 1))
            self.search_term = df.format("F Y")
        if day:
            self.entries = self.entries.filter(date__day=day)
            self.search_term = date_format(date(int(year), int(month), int(day)))
        return Page.serve(self, request, *args, **kwargs)

    @route(r"^tag/(?P<tag>[-\w]+)/$")
    def entries_by_tag(self, request, tag, *args, **kwargs):
        from puput.models import Tag

        self.search_type = _("tag")
        object_or_slug = get_object_or_None(Tag, slug=tag) or tag
        self.search_term = str(object_or_slug)
        self.entries = self.get_entries().filter(tags__slug=tag)
        return Page.serve(self, request, *args, **kwargs)

    @route(r"^category/(?P<category>[-\w]+)/$")
    def entries_by_category(self, request, category, *args, **kwargs):
        from puput.models import Category

        self.search_type = _("category")
        object_or_slug = get_object_or_None(Category, slug=category) or category
        self.search_term = str(object_or_slug)
        self.entries = self.get_entries().filter(entry_categories__category__slug=category)
        return Page.serve(self, request, *args, **kwargs)

    @route(r"^author/(?P<author>%s)/$" % USERNAME_REGEX)
    def entries_by_author(self, request, author, *args, **kwargs):
        self.search_type = _("author")
        object_or_slug = get_object_or_None(get_user_model(), **{USERNAME_FIELD: author}) or author
        self.search_term = str(object_or_slug)
        self.entries = self.get_entries().filter(**{"owner__%s" % USERNAME_FIELD: author})
        return Page.serve(self, request, *args, **kwargs)

    @route(r"^search/$")
    def entries_search(self, request, *args, **kwargs):
        search_query = request.GET.get("q", None)
        self.entries = self.get_entries()
        if search_query:
            self.entries = self.entries.search(search_query)
            self.search_term = search_query
            self.search_type = _("search")
            Query.get(search_query).add_hit()
        return Page.serve(self, request, *args, **kwargs)

    @route(r"^$")
    def entries_list(self, request, *args, **kwargs):
        self.entries = self.get_entries()
        return Page.serve(self, request, *args, **kwargs)



import markdown
from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.template.defaultfilters import slugify
from django.utils.translation import gettext_lazy as _

from wagtail.models import Page
from wagtail.admin.panels import FieldPanel
from wagtail.snippets.models import register_snippet
from wagtail.search import index
from taggit.models import TaggedItemBase, Tag as TaggitTag
from modelcluster.fields import ParentalKey

from .abstracts import EntryAbstract, BlogAbstract
from .utils import import_model
from .routes import BlogRoutes
from .managers import TagManager, CategoryManager, BlogManager

Entry = import_model(getattr(settings, "PUPUT_ENTRY_MODEL", EntryAbstract))
Blog = import_model(getattr(settings, "PUPUT_BLOG_MODEL", BlogAbstract))


class BlogPage(BlogRoutes, Page, Blog):
    extra = BlogManager()

    content_panels = Page.content_panels + getattr(Blog, "content_panels", [])
    settings_panels = Page.settings_panels + getattr(Blog, "settings_panels", [])

    subpage_types = ["puput.EntryPage"]

    def get_entries(self):
        return EntryPage.objects.descendant_of(self).live().order_by("-date").select_related("owner")

    def get_context(self, request, *args, **kwargs):
        context = super(BlogPage, self).get_context(request, *args, **kwargs)
        context["entries"] = self.entries
        context["blog_page"] = self
        context["search_type"] = getattr(self, "search_type", "")
        context["search_term"] = getattr(self, "search_term", "")
        return context

    @property
    def last_url_part(self):
        """
        Get the BlogPage url without the domain
        """
        return self.get_url_parts()[-1]

    class Meta:
        verbose_name = _("Blog")


@register_snippet
class Category(models.Model):
    name = models.CharField(max_length=80, unique=True, verbose_name=_("Category name"))
    slug = models.SlugField(unique=True, max_length=80)
    parent = models.ForeignKey(
        "self",
        blank=True,
        null=True,
        related_name="children",
        verbose_name=_("Parent category"),
        on_delete=models.SET_NULL,
    )
    description = models.CharField(max_length=500, blank=True, verbose_name=_("Description"))

    objects = CategoryManager()

    def __str__(self):
        return self.name

    def clean(self):
        if self.parent:
            parent = self.parent
            if self.parent == self:
                raise ValidationError(_("Parent category cannot be self."))
            if parent.parent and parent.parent == self:
                raise ValidationError(_("Cannot have circular Parents."))

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.name)
        return super(Category, self).save(*args, **kwargs)

    class Meta:
        ordering = ["name"]
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")


class CategoryEntryPage(models.Model):
    category = models.ForeignKey(Category, related_name="+", verbose_name=_("Category"), on_delete=models.CASCADE)
    page = ParentalKey("EntryPage", related_name="entry_categories")
    panels = [FieldPanel("category")]

    def __str__(self):
        return str(self.category)


class TagEntryPage(TaggedItemBase):
    content_object = ParentalKey("EntryPage", related_name="entry_tags")


@register_snippet
class Tag(TaggitTag):
    objects = TagManager()

    class Meta:
        proxy = True


class EntryPageRelated(models.Model):
    entrypage_from = ParentalKey("EntryPage", verbose_name=_("Entry"), related_name="related_entrypage_from")
    entrypage_to = ParentalKey("EntryPage", verbose_name=_("Entry"), related_name="related_entrypage_to")

    def __str__(self):
        return str(self.entrypage_to)


class EntryPage(Entry, Page):
    # Search
    search_fields = Page.search_fields + [
        index.SearchField("body"),
        index.SearchField("excerpt"),
        index.FilterField("page_ptr_id"),
    ]

    # Panels
    content_panels = getattr(Entry, "content_panels", [])

    promote_panels = Page.promote_panels + getattr(Entry, "promote_panels", [])

    settings_panels = (
        Page.settings_panels
        + [
            FieldPanel("date"),
            FieldPanel("owner"),
        ]
        + getattr(Entry, "settings_panels", [])
    )

    # Parent and child settings
    parent_page_types = ["puput.BlogPage"]
    subpage_types = []

    def get_sitemap_urls(self, request=None):
        from .urls import get_entry_url

        root_url = self.get_url_parts()[1]
        entry_url = get_entry_url(self, self.blog_page.page_ptr, root_url)
        return [
            {
                "location": root_url + entry_url,
                # fall back on latest_revision_created_at if last_published_at is null
                # (for backwards compatibility from before last_published_at was added)
                "lastmod": (self.last_published_at or self.latest_revision_created_at),
            }
        ]

    @property
    def blog_page(self):
        return self.get_parent().specific

    @property
    def related(self):
        return [related.entrypage_to for related in self.related_entrypage_from.all()]

    @property
    def has_related(self):
        return self.related_entrypage_from.count() > 0

    def get_absolute_url(self):
        return self.full_url

    def get_context(self, request, *args, **kwargs):
        context = super(EntryPage, self).get_context(request, *args, **kwargs)
        context["blog_page"] = self.blog_page
        return context

    def get_preview_context(self, request, mode_name):
        preview_context = super().get_preview_context(request, mode_name)

        if self.markdown_body:
            preview_context["page"].body = markdown.markdown(self.markdown_body, extensions=["extra", "codehilite"])

        return preview_context

    class Meta:
        verbose_name = _("Entry")
        verbose_name_plural = _("Entries")



from django.template import Library
from django.urls import resolve
from django.template.defaultfilters import urlencode
from django.template.loader import render_to_string
from django_social_share.templatetags.social_share import _build_url
from el_pagination.templatetags.el_pagination_tags import show_pages, paginate

from ..conf import settings
from ..utils import import_model
from ..models import Category, Tag

from wagtail.models import Site

register = Library()


@register.inclusion_tag("puput/tags/entries_list.html", takes_context=True)
def recent_entries(context, limit=None):
    blog_page = context["blog_page"]
    entries = blog_page.get_entries().order_by("-date")
    if limit:
        entries = entries[:limit]
    context["entries"] = entries
    return context


@register.inclusion_tag("puput/tags/entries_list.html", takes_context=True)
def popular_entries(context, limit=None):
    blog_page = context["blog_page"]
    entries = blog_page.get_entries().order_by("-num_comments", "-date")
    if limit:
        entries = entries[:limit]
    context["entries"] = entries
    return context


@register.inclusion_tag("puput/tags/tags_list.html", takes_context=True)
def tags_list(context, limit=None, tags_qs=None):
    blog_page = context["blog_page"]
    if tags_qs:
        tags = tags_qs.all()
    else:
        tags = Tag.objects.most_common(blog_page)
    if limit:
        tags = tags[:limit]
    context["tags"] = tags
    return context


@register.inclusion_tag("puput/tags/categories_list.html", takes_context=True)
def categories_list(context, categories_qs=None):
    blog_page = context["blog_page"]
    if categories_qs:
        categories = categories_qs.all()
    else:
        categories = Category.objects.with_uses(blog_page).filter(parent=None)
    context["categories"] = categories
    return context


@register.inclusion_tag("puput/tags/archives_list.html", takes_context=True)
def archives_list(context):
    blog_page = context["blog_page"]
    context["archives"] = blog_page.get_entries().datetimes("date", "day", order="DESC")
    return context


@register.simple_tag(takes_context=True)
def entry_url(context, entry, blog_page):
    from ..urls import get_entry_url

    return get_entry_url(entry, blog_page.page_ptr, Site.find_for_request(context["request"]).root_page)


@register.simple_tag(takes_context=True)
def canonical_url(context, entry=None):
    if entry and resolve(context.request.path_info).url_name == "wagtail_serve":
        return context.request.build_absolute_uri(entry_url(context, entry, entry.blog_page))
    return context.request.build_absolute_uri()


@register.simple_tag(takes_context=True)
def image_url(context, url):
    return context.request.build_absolute_uri(url)


@register.simple_tag(takes_context=True)
def feeds_url(context, blog_page):
    from ..urls import get_feeds_url

    return get_feeds_url(blog_page.page_ptr, Site.find_for_request(context["request"]).root_page)


@register.simple_tag(takes_context=True)
def show_comments(context):
    blog_page = context["blog_page"]
    entry = context["self"]
    if blog_page.display_comments:
        try:
            comment_class = import_model(settings.PUPUT_COMMENTS_PROVIDER)(blog_page, entry)
            comment_context = comment_class.get_context()
            comment_context.update(context.flatten())
            return render_to_string(comment_class.template, context=comment_context)
        except AttributeError:
            raise Exception("To use comments you need to specify " "PUPUT_COMMENTS_PROVIDER in settings.")
    return ""


# Avoid to import endless_pagination in installed_apps and in the templates
register.tag("show_paginator", show_pages)
register.tag("paginate", paginate)


@register.simple_tag(takes_context=True)
def post_to_linkendin_url(context, obj_or_url=None):
    request = context.get("request")
    if request:
        url = _build_url(request, obj_or_url)
        context["linkendin_url"] = "https://www.linkedin.com/shareArticle?url={}".format(urlencode(url))
    return context


@register.inclusion_tag("puput/tags/post_to_linkedin.html", takes_context=True)
def post_to_linkendin(context, obj_or_url=None, link_text="Post to LinkedIn"):
    context = post_to_linkendin_url(context, obj_or_url)
    context["link_text"] = link_text
    return context



# Generated by Django 3.2.16 on 2022-12-22 11:31

from django.db import migrations
import wagtailmarkdown.fields


class Migration(migrations.Migration):

    dependencies = [
        ('puput', '0005_blogpage_main_color'),
    ]

    operations = [
        migrations.AddField(
            model_name='entrypage',
            name='markdown_body',
            field=wagtailmarkdown.fields.MarkdownField(blank=True, null=True, verbose_name='body (Markdown)'),
        ),
    ]



from django.conf import settings
from django.http import Http404, HttpResponse
from django.views.generic import View
from django.views.decorators.csrf import ensure_csrf_cookie
from django.utils.decorators import method_decorator

from wagtail import hooks
from wagtail.models import Site

from .models import EntryPage
from .utils import strip_prefix_and_ending_slash, import_model


class EntryPageServe(View):
    """
    This class is responsible to serve entries with a proper blog url format:
    http://wwww.example.com/2015/10/01/my-first-post

    If you set your blog as Wagtail Root page, the url is like the above example.
    Otherwise if you have a multiple blog instances, you need to pass the slug of the blog
    page instance that you want to use:
    http://wwww.example.com/weblog/2015/10/01/my-first-post
    http://wwww.example.com/videblog/2015/10/01/my-first-video
    """

    @method_decorator(ensure_csrf_cookie)
    def get(self, request, *args, **kwargs):
        site = Site.find_for_request(request)
        if not site:
            raise Http404
        if request.resolver_match.url_name == "entry_page_serve_slug":
            # Splitting the request path and obtaining the path_components
            # this way allows you to place the blog at the level you want on
            # your sitemap.
            # Example:
            # splited_path =  ['es', 'blog', '2016', '06', '23', 'blog-entry']
            # slicing this way you obtain:
            # path_components =  ['es', 'blog', 'blog-entry']
            # with the oldest solution you'll get ['es', 'blog-entry']
            # and a 404 will be raised
            splited_path = strip_prefix_and_ending_slash(request.path).split("/")
            path_components = splited_path[:-4] + splited_path[-1:]
        else:
            path_components = [strip_prefix_and_ending_slash(request.path).split("/")[-1]]
        page, args, kwargs = site.root_page.specific.route(request, path_components)

        for fn in hooks.get_hooks("before_serve_page"):
            result = fn(page, request, args, kwargs)
            if isinstance(result, HttpResponse):
                return result
        return page.serve(request, *args, **kwargs)


class EntryPageUpdateCommentsView(View):
    def post(self, request, entry_page_id, *args, **kwargs):
        try:
            entry_page = EntryPage.objects.get(pk=entry_page_id)
            blog_page = entry_page.blog_page
            comment_class = import_model(settings.PUPUT_COMMENTS_PROVIDER)(blog_page, entry_page)
            num_comments = comment_class.get_num_comments()
            entry_page.num_comments = num_comments
            entry_page.save(update_fields=("num_comments",))
            return HttpResponse()
        except EntryPage.DoesNotExist:
            raise Http404



# Generated by Django 3.2.3 on 2023-05-12 11:34

from django.db import migrations
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('puput', '0006_entrypage_markdown_body'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entrypage',
            name='body',
            field=wagtail.fields.RichTextField(blank=True, null=True, verbose_name='body'),
        ),
    ]



from django.conf import settings
from django.urls import reverse, path, include

from .feeds import BlogPageFeed
from .views import EntryPageServe, EntryPageUpdateCommentsView
from .utils import strip_prefix_and_ending_slash


urlpatterns = [
    path(
        route="entry_page/<entry_page_id>/update_comments/",
        view=EntryPageUpdateCommentsView.as_view(),
        name="entry_page_update_comments",
    ),
    path(
        route="<path:blog_path>/<int:year>/<int:month>/<int:day>/<str:slug>/",
        view=EntryPageServe.as_view(),
        name="entry_page_serve_slug",
    ),
    path(route="<int:year>/<int:month>/<int:day>/<str:slug>/", view=EntryPageServe.as_view(), name="entry_page_serve"),
    path(route="<path:blog_path>/feed/", view=BlogPageFeed(), name="blog_page_feed_slug"),
    path(route="feed/", view=BlogPageFeed(), name="blog_page_feed"),
]

if not getattr(settings, "PUPUT_AS_PLUGIN", False):
    from wagtail import urls as wagtail_urls
    from wagtail.admin import urls as wagtailadmin_urls
    from wagtail.documents import urls as wagtaildocs_urls
    from wagtail.contrib.sitemaps.views import sitemap

    urlpatterns.extend(
        [
            path(route="blog_admin/", view=include(wagtailadmin_urls)),
            path(route="", view=include(wagtail_urls)),
            path(route="documents/", view=include(wagtaildocs_urls)),
            path(route="sitemap.xml", view=sitemap),
        ]
    )


def get_entry_url(entry, blog_page, root_page):
    """
    Get the entry url given and entry page a blog page instances.
    It will use an url or another depending if blog_page is the root page.
    """
    if root_page == blog_page:
        return reverse(
            "entry_page_serve",
            kwargs={
                "year": entry.date.strftime("%Y"),
                "month": entry.date.strftime("%m"),
                "day": entry.date.strftime("%d"),
                "slug": entry.slug,
            },
        )
    else:
        # The method get_url_parts provides a tuple with a custom URL routing
        # scheme. In the last position it finds the subdomain of the blog, which
        # it is used to construct the entry url.
        # Using the stripped subdomain it allows Puput to generate the urls for
        # every sitemap level
        blog_path = strip_prefix_and_ending_slash(blog_page.specific.last_url_part)
        return reverse(
            "entry_page_serve_slug",
            kwargs={
                "blog_path": blog_path,
                "year": entry.date.strftime("%Y"),
                "month": entry.date.strftime("%m"),
                "day": entry.date.strftime("%d"),
                "slug": entry.slug,
            },
        )


def get_feeds_url(blog_page, root_page):
    """
    Get the feeds urls a blog page instance.
    It will use an url or another depending if blog_page is the root page.
    """
    if root_page == blog_page:
        return reverse("blog_page_feed")
    else:
        blog_path = strip_prefix_and_ending_slash(blog_page.specific.last_url_part)
        return reverse("blog_page_feed_slug", kwargs={"blog_path": blog_path})



from mimetypes import guess_type
from urllib.parse import urljoin

from django import http
from django.contrib.syndication.views import Feed
from django.utils.feedgenerator import Rss201rev2Feed
from django.template.defaultfilters import truncatewords_html
from wagtail.models import Site

from .models import BlogPage


class BlogPageFeedGenerator(Rss201rev2Feed):
    def add_root_elements(self, handler):
        super(BlogPageFeedGenerator, self).add_root_elements(handler)
        if self.feed["image_link"]:
            handler.addQuickElement(
                "image",
                "",
                {
                    "url": self.feed["image_link"],
                    "title": self.feed["title"],
                    "link": self.feed["link"],
                },
            )


class BlogPageFeed(Feed):
    feed_type = BlogPageFeedGenerator

    def __call__(self, request, *args, **kwargs):
        if request.resolver_match.url_name == "blog_page_feed_slug":
            self.blog_page = BlogPage.extra.get_by_path(kwargs["blog_path"])
            if not self.blog_page:
                raise http.Http404
        else:
            self.blog_page = BlogPage.objects.first()
        self.request = request
        return super(BlogPageFeed, self).__call__(request, *args, **kwargs)

    def title(self):
        return self.blog_page.title

    def description(self):
        return self.blog_page.description

    def link(self):
        return self.blog_page.last_url_part

    def items(self):
        return self.blog_page.get_entries()[:20]

    def item_title(self, item):
        return item.title

    def _item_short_description(self, item):
        if item.excerpt and item.excerpt.strip() != "":
            return item.excerpt
        else:
            return truncatewords_html(item.body, 70)

    def item_description(self, item):
        if self.blog_page.short_feed_description:
            return self._item_short_description(item)
        return item.body

    def item_pubdate(self, item):
        return item.date

    def item_link(self, item):
        from .urls import get_entry_url

        entry_url = get_entry_url(item, self.blog_page.page_ptr, Site.find_for_request(self.request).root_page)
        return self.request.build_absolute_uri(entry_url)

    def item_enclosure_url(self, item):
        if item.header_image:
            site = Site.find_for_request(self.request)
            return urljoin(site.root_url, item.header_image.file.url)
        return None

    def item_enclosure_mime_type(self, item):
        if item.header_image:
            mime, enc = guess_type(self.item_enclosure_url(item))
            return mime
        return None

    def item_enclosure_length(self, item):
        if item.header_image:
            return item.header_image.file.size
        return 0

    def _channel_image_link(self):
        if self.blog_page.header_image:
            site = Site.find_for_request(self.request)
            return urljoin(site.root_url, self.blog_page.header_image.file.url)

    def feed_extra_kwargs(self, obj):
        return {"image_link": self._channel_image_link()}
